package cn.com.ty.chat.service.impl;

import cn.com.ty.chat.common.SensitivewordFilter;
import cn.com.ty.chat.mapper.SensitiveWordsMapper;
import cn.com.ty.chat.mapper.entity.SensitiveWords;
import cn.com.ty.chat.service.ISensitiveWordsService;
import cn.com.ty.chat.utils.RedisUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 敏感词汇 -ServiceImpl
 *
 * @author Xson
 * @date 2020-04-24
 */
@Service
@Transactional
public class SensitiveWordsServiceImpl extends ServiceImpl<SensitiveWordsMapper, SensitiveWords> implements ISensitiveWordsService {
    @Autowired
    private RedisUtils redis ;
    @Override
    public Set<String> changText(String text, Integer matchType) {
        Set<String> sensitiveWordList = new HashSet<String>();
        for (int i = 0; i < text.length(); i++) {
            int length = CheckSensitiveWords(text, i, matchType); // 判断是否包含敏感字符
            if (length > 0) { // 存在,加入list中
                sensitiveWordList.add(text.substring(i, i + length));
                i = i + length - 1; // 减1的原因，是因为for会自增
            }
        }

        return sensitiveWordList;
    }
    private int CheckSensitiveWords(String txt, int beginIndex, int matchType) {
        boolean flag = false; // 敏感词结束标识位：用于敏感词只有1位的情况
        int matchFlag = 0; // 匹配标识数默认为0
        char word = 0;

        String sensitiveWords  = redis.get("sensitiveWords");
        List<SensitiveWords> all = null;
        if(StringUtils.isBlank(sensitiveWords)){
            all = baseMapper.getAll();
        }else{
            all = JSONObject.parseObject(redis.get("sensitiveWords"),List.class);
        }
        Set<String> keyWordSet = new HashSet<>();
        for (SensitiveWords s : all) {
            keyWordSet.add(s.getKeyword());
        }
        Map nowMap = addSensitiveWordToHashMap(keyWordSet);
        Map mapByKey = null;
        for (int i = beginIndex; i < txt.length(); i++) {
            word = txt.charAt(i);
//            System.out.println(nowMap);
            if (mapByKey == null) {
                mapByKey = (Map)nowMap.get(word); // 获取指定key
            } else {
                mapByKey = (Map)mapByKey.get(word);
            }
//            System.out.println(mapByKey);
            if (mapByKey != null) { // 存在，则判断是否为最后一个
                matchFlag++; // 找到相应key，匹配标识+1
                if ("1".equals(mapByKey.get("isEnd"))) { // 如果为最后一个匹配规则,结束循环，返回匹配标识数
                    flag = true; // 结束标志位为true
                    if (SensitivewordFilter.minMatchTYpe == matchType) { // 最小规则，直接返回,最大规则还需继续查找
                        break;
                    }
                }
            } else { // 不存在，直接返回
                break;
            }
        }
        if (matchFlag < 1 || !flag) { // 长度必须大于等于1，为词
            matchFlag = 0;
        }
        return matchFlag;
    }

    private Map addSensitiveWordToHashMap(Set<String> keyWordSet) {
        Map sensitiveWordMap = new HashMap(keyWordSet.size()); // 初始化敏感词容器，减少扩容操作
        String key = null;
        Map nowMap = null;
        Map<String, String> newWorMap = null;
        // 迭代keyWordSet
        Iterator<String> iterator = keyWordSet.iterator();
        while (iterator.hasNext()) {
            key = iterator.next(); // 关键字
            nowMap = sensitiveWordMap;
            for (int i = 0; i < key.length(); i++) {
                char keyChar = key.charAt(i); // 转换成char型
                Object wordMap = nowMap.get(keyChar); // 获取

                if (wordMap != null) { // 如果存在该key，直接赋值
                    nowMap = (Map)wordMap;
                } else { // 不存在则，则构建一个map，同时将isEnd设置为0，因为他不是最后一个
                    newWorMap = new HashMap<String, String>();
                    newWorMap.put("isEnd", "0"); // 不是最后一个
                    nowMap.put(keyChar, newWorMap);
                    nowMap = newWorMap;
                }

                if (i == key.length() - 1) {
                    nowMap.put("isEnd", "1"); // 最后一个
                }
            }
        }
        return sensitiveWordMap;
    }
}
