package cn.com.ty.lift.ud.batchUtils;

import cn.com.ty.lift.common.aliservice.aliyunoss.AliyunOSS;
import cn.com.ty.lift.ud.batchUtils.req.BrandLibraryBatchUploadReq;
import cn.com.ty.lift.ud.dataBank.mapper.entity.LiftBrandEntity;
import cn.com.ty.lift.ud.dataBank.mapper.entity.LiftCategoryEntity;
import cn.com.ty.lift.ud.dataBank.mapper.entity.LiftFileEntity;
import cn.com.ty.lift.ud.dataBank.service.ILiftBrandService;
import cn.com.ty.lift.ud.dataBank.service.ILiftCategoryService;
import cn.com.ty.lift.ud.dataBank.service.ILiftFileService;
import cn.com.ty.lift.ud.exception.CatchException;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.sound.midi.Soundbank;
import java.util.*;

@RestController
@RequestMapping("/api/batch-file")
public class BrandLibraryBatchUploadUtil {
    @Autowired
    private ILiftBrandService brandService;
    @Autowired
    private ILiftCategoryService categoryService;
    @Autowired
    private ILiftFileService liftFileService;

    static final List<String> parantCategory = Arrays.asList("直梯", "扶梯");
    static final List<String> subCategory = Arrays.asList("使用手册", "光幕", "图纸", "培训", "安装工艺", "维修资料", "调试手册及资料", "门机", "其他");
    static final List<String> subCategoryIcon = Arrays.asList("https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/使用手册.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/光幕.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/图纸.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/培训.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/安装工艺.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/维修及维修资料.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/调试手册及资料.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/门机.png",
            "https://ty-oss-file.oss-cn-hangzhou.aliyuncs.com/library_category_icon/其他资料.png");

    @GetMapping("initCategory")
    public void initCategory(@RequestParam String id) {
        try {
            LiftBrandEntity brandEntity = Optional
                    .ofNullable(brandService.getOne(new QueryWrapper<LiftBrandEntity>()
                            .eq("id", id)))
                    .orElseThrow(() -> new CatchException("该品牌不存在，请重启后重新输入"));
            parantCategory.forEach(parantS -> {
                LiftCategoryEntity category = new LiftCategoryEntity();
                category.setName(parantS);
                category.setBrandId(brandEntity.getId());
                category.setPid(0L);
                category.setStatuz(1);
                category.setSort("" + parantCategory.indexOf(parantS));
                category.setCreateBy("aaaaaa");
                categoryService.save(category);
                subCategory.forEach(subS -> {
                    LiftCategoryEntity sub = new LiftCategoryEntity();
                    sub.setName(subS);
                    sub.setBrandId(brandEntity.getId());
                    sub.setPid(category.getId());
                    sub.setStatuz(1);
                    sub.setIcon(subCategoryIcon.get(subCategory.indexOf(subS)));
                    sub.setSort("" + subCategory.indexOf(subS));
                    sub.setCreateBy("aaaaaa");
                    categoryService.save(sub);
                });
            });
        } catch (CatchException e) {
            e.printStackTrace();
        } finally {
        }
    }

    @GetMapping("batchFile")
    public RestResponse batch(@RequestBody BrandLibraryBatchUploadReq param) {
        List<String> rtn = new ArrayList<>();
        try {
            //"电梯资料[2021]/电梯资料2021/东芝系列资料/直梯/使用手册/"
            LiftBrandEntity brandEntity = Optional.ofNullable(brandService.getById(param.getBrandId())).orElseThrow(() -> new CatchException("该品牌不存在，请重启后重新输入"));
            List<String> filesUrl = Optional
                    .ofNullable(AliyunOSS.me().listObjects("ty-oss-file", param.getOssDir()  + "/"))
                    .orElseThrow(() -> new CatchException(param.getOssDir() + "-->不存在"));

            for (String fileUrl : filesUrl) {
                Optional<LiftFileEntity> liftFileEntityOptional = Optional.ofNullable(liftFileService.getOne(new QueryWrapper<LiftFileEntity>()
                        .eq("name", fileUrl.substring(fileUrl.lastIndexOf("/") + 1))
                        .eq("brand_id", param.getBrandId())
                        .eq("data_table", 2)
                        .eq("catagory_id", param.getCategoryId())
                ));
                if (!liftFileEntityOptional.isPresent()){
                    LiftFileEntity fileEntity = new LiftFileEntity();
                    fileEntity.setUrl(fileUrl);
                    fileEntity.setName(fileUrl.substring(fileUrl.lastIndexOf("/") + 1));
                    fileEntity.setBrandId(brandEntity.getId().intValue());
                    fileEntity.setDataId(brandEntity.getId().intValue());
                    fileEntity.setDataTable(2);
                    fileEntity.setStatuz(1);
                    fileEntity.setSort(filesUrl.indexOf(fileUrl));
                    fileEntity.setCreateBy("aaaaaa");
                    fileEntity.setCatagoryId(Integer.parseInt(param.getCategoryId()));
                    liftFileService.save(fileEntity);

                    rtn.add(fileUrl);
                }
            }
            return RestResponse.success(rtn);
        } catch (CatchException e) {
            return RestResponse.fail(e.getMsg());
        }
    }
}
